#pragma semicolon 1

#include <sourcemod>
#include <loghelper>
#include <colors>

#include <economy>

const DEFAULT_MONEY_AMOUNT = 0;

/** 
 * Global Enums
 */
enum Category
{
	String:CategoryId[64],
	String:CategoryName[64],
	String:CategoryDescription[128]
}

enum Item
{
	String:ItemCategoryId[64],
	String:ItemId[64],
	String:ItemName[64],
	String:ItemDescription[128],
	Handle:ItemCallbackPlugin,
	ItemUseCallback:ItemCallback,
	ItemPrice,
	bool:IsItemToggleable,
	bool:IsItemFinite
}

/**
 * Global Variables
 */
new reconnectcounter = 0;
new databaseCounter[MAXPLAYERS+1];

new Handle:hSQL = INVALID_HANDLE;

new String:lastCategoryId[64][MAXPLAYERS+1];

new categories[1024][Category]; 
new categoryCount = 0;

new items[1024][Item]; 
new itemCount = 0;

new bool:isLoading = false;

new String:howToDonate[255][512];
new howToDonateLinesCount = 0;

public Plugin:myinfo =
{
	name		= "[Economy] Core",
	author	  	= "alongub",
	description = "Core component for [Economy]",
	version	 	= PL_VERSION,
	url		 	= "http://steamcommunity.com/id/alon"
};

public OnPluginStart()
{	
	RegConsoleCmd("say", Command_Say);
	RegConsoleCmd("say_team", Command_Say);
	
	RegAdminCmd("sm_reloadeconomy",
		Command_ReloadEconomy,
		ADMFLAG_KICK,
		"Reloads economy system.");
        
	RegAdminCmd("sm_givecredits",
		Command_GiveCredits,
		ADMFLAG_KICK,
		"Give credits.");        
}

public OnMapStart()
{	
	reconnectcounter = 0;
	ConnectSQL();
	LoadHowToDonate();	
}

public OnMapEnd()
{
	categoryCount = 0;
	itemCount = 0;
}

public APLRes:AskPluginLoad2(Handle:myself, bool:late, String:error[], err_max)
{
	CreateNative("Economy_GivePlayerMoney", Native_GivePlayerMoney);
	CreateNative("Economy_GetPlayerMoney", Native_GetPlayerMoney);
	CreateNative("Economy_GivePlayerItem", Native_GivePlayerItem);

	CreateNative("Economy_RegisterItem", Native_RegisterItem);
	CreateNative("Economy_RegisterItemCategory", Native_RegisterItemCategory);

	CreateNative("Economy_GetPrimaryToggled", Native_GetPrimaryToggled);
	CreateNative("Economy_IsToggled", Native_IsToggled);
	CreateNative("Economy_ToggleItem", Native_ToggleItem);
	CreateNative("Economy_ToggleItemCategory", Native_ToggleItemCategory);
	
	RegPluginLibrary("economy");
	
	return APLRes_Success;
}

public OnClientPostAdminCheck(client)
{
	if (hSQL != INVALID_HANDLE && IsValidPlayer(client))
		CreateUser(client);	
}

public Action:Command_ReloadEconomy(client, args)
{
	reconnectcounter = 0;
	isLoading = false;
	
	ConnectSQL();
}

LoadHowToDonate()
{
	new String:path[PLATFORM_MAX_PATH];
	BuildPath(Path_SM, path, PLATFORM_MAX_PATH, "configs/store/howtodonate.txt");
			
	new Handle:file = OpenFile(path, "r");
	
	decl String:buffer[255];
	
	while (!IsEndOfFile(file) && ReadFileLine(file, buffer, sizeof(buffer)))
	{
		TrimString(buffer);
		
		if (buffer[0] == '\0' || buffer[0] == ';' || (buffer[0] == '/' && buffer[1] == '/')) 
		{
			continue;
		}

		strcopy(howToDonate[howToDonateLinesCount], 255, buffer);
		howToDonateLinesCount++;
	}
	
	CloseHandle(file);
}

RegisterItemCategory(String:_id[], String:_name[], String:_description[])
{
	new j = categoryCount;
	
	for (new category = 0; category < categoryCount+1; category++)
	{
		if (StrEqual(categories[category][CategoryId], _id))
		{
			j = category;
			break;
		}
	}
	
	strcopy(categories[j][CategoryId], 64, _id); 
	strcopy(categories[j][CategoryName], 64, _name); 
	strcopy(categories[j][CategoryDescription], 64, _description); 
	
	if (j == categoryCount)
		categoryCount++;
}

RegisterItem(String:categoryId[], String:id[], String:name[], String:description[], Handle:plugin, ItemUseCallback:callback, price, bool:isFinite, bool:isTogglable)
{
	new j = itemCount;
	
	for (new item = 0; item < itemCount; item++)
	{
		if (StrEqual(items[item][ItemId], id))
		{
			j = item;
			break;
		}
	}
	
	strcopy(items[j][ItemCategoryId], 64, categoryId); 
	strcopy(items[j][ItemId], 64, id); 
	strcopy(items[j][ItemName], 64, name); 
	strcopy(items[j][ItemDescription], 64, description); 
	items[j][ItemCallbackPlugin] = plugin;
	items[j][ItemCallback] = callback;
	items[j][ItemPrice] = price;
	items[j][IsItemFinite] = isFinite;
	items[j][IsItemToggleable] = isTogglable;
	
	if (j == itemCount)
		itemCount++;
}

bool:GetPrimaryToggled(client, String:categoryId[], String:itemId[])
{
	if (hSQL == INVALID_HANDLE)
	{
		ConnectSQL();
		return false;
	}
	
	decl String:auth[32];
	GetClientAuthString(client, auth, sizeof(auth));

	decl String:query[255];
	Format(query, sizeof(query), "SELECT item_id FROM playeritem WHERE isToggled = 1 AND auth = '%s'", auth);
	
	SQL_LockDatabase(hSQL);

	new Handle:hQuery = SQL_Query(hSQL, query);
	
	if (hQuery == INVALID_HANDLE)
	{
		SQL_UnlockDatabase(hSQL);
		return false;
	}

	SQL_UnlockDatabase(hSQL); 
	
	while (SQL_FetchRow(hQuery))
	{		
		SQL_FetchString(hQuery, 0, itemId, 64);
		
		for (new item = 0; item < itemCount; item++)
		{
			if (StrEqual(items[item][ItemId], itemId) && StrEqual(items[item][ItemCategoryId], categoryId))
			{
				return true;
			}
		}
	}
	
	return false;
}

bool:IsToggled(client, String:item_id[])
{
	if (hSQL == INVALID_HANDLE)
	{
		ConnectSQL();
		return false;
	}
	
	decl String:auth[32];
	GetClientAuthString(client, auth, sizeof(auth));

	decl String:query[255];
	Format(query, sizeof(query), "SELECT isToggled FROM playeritem WHERE item_id = '%s' AND auth = '%s'", item_id, auth);
	
	SQL_LockDatabase(hSQL);

	new Handle:hQuery = SQL_Query(hSQL, query);
	
	if (hQuery == INVALID_HANDLE)
	{
		SQL_UnlockDatabase(hSQL);
		return false;
	}

	SQL_UnlockDatabase(hSQL); 

	SQL_FetchRow(hQuery);

	return (SQL_FetchInt(hQuery, 0) == 1);
}

ToggleItem(client, String:item_id[], bool:value)
{
	if (hSQL == INVALID_HANDLE)
	{
		ConnectSQL();
		return false;
	}
	
	decl String:auth[32];
	GetClientAuthString(client, auth, sizeof(auth));

	decl String:query[255];
	Format(query, sizeof(query), "UPDATE playeritem SET isToggled = %d WHERE item_id = '%s' AND auth = '%s'", (value ? 1 : 0), item_id, auth);
	
	SQL_LockDatabase(hSQL);

	new Handle:hQuery = SQL_Query(hSQL, query);
	
	if (hQuery == INVALID_HANDLE)
	{
		SQL_UnlockDatabase(hSQL);
		return false;
	}

	SQL_UnlockDatabase(hSQL); 
	
	return true;
}

ToggleItemCategory(client, String:category_id[], bool:value)
{
	for (new item = 0; item < itemCount; item++)
	{
		if (StrEqual(items[item][ItemCategoryId], category_id))
			ToggleItem(client, items[item][ItemId], value);
	}
}

ConnectSQL()
{
	if (isLoading)
		return;
		
	isLoading = true;
	
	if (hSQL != INVALID_HANDLE)
		CloseHandle(hSQL);
	
	hSQL = INVALID_HANDLE;

	if (SQL_CheckConfig("economy"))
	{
		SQL_TConnect(ConnectSQLCallback, "economy");
	}
	else
	{
		PrintToServer("No config entry found for 'economy' in databases.cfg");
		isLoading = false;
	}
}
 
public ConnectSQLCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (reconnectcounter >= 5)
	{
		LogError("PLUGIN STOPPED - reconnect counter reached max - PLUGIN STOPPED");
		return -1;
	}

	if (hndl == INVALID_HANDLE)
	{
		LogError("Connection to SQL database has failed, Reason: %s", error);
		reconnectcounter++;
		ConnectSQL();

		return -1;
	}

	new String:driver[16];
	SQL_GetDriverIdent(owner,driver,sizeof(driver));
	
	if (StrEqual(driver,"mysql",false))
		SQL_FastQuery(hndl, "SET NAMES 'utf8'");

	// save database handle
	hSQL = CloneHandle(hndl);
	isLoading = false;	
	reconnectcounter = 1;
	return 1;
}

public CreateSQLTableCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE)
	{
		LogError("Connection to SQL database has failed, Reason: %s", error);
		reconnectcounter++;
		ConnectSQL();
		return -1;
	}
	
	if (hndl == INVALID_HANDLE)
	{
		LogError("Could not create the necessary table in database, Reason: %s", error);
		return -1;
	}

	return 1;
}

public Action:Command_Say(client, argc)
{
	decl String:args[192];
	GetCmdArgString(args, sizeof(args));

	StripQuotes(args);
		
	if (StrEqual(args, "!store", false))
	{
		MainMenu(client);
	}
	else if (StrEqual(args, "!inv", false))
	{
		OpenInventory(client);
	}
	
	return Plugin_Continue;
}

public Action:Command_GiveCredits(client, args)
{
	if (args < 2)
	{
		ReplyToCommand(client, "Usage: sm_givecredits <name> <credits>");
		return Plugin_Handled;
	}
    
	decl String:target[65];
	decl String:target_name[MAX_TARGET_LENGTH];
	decl target_list[MAXPLAYERS];
	decl target_count;
	decl bool:tn_is_ml;
    
	GetCmdArg(1, target, sizeof(target));
    
	new String:money[32];
	GetCmdArg(2, money, sizeof(money));
    
	new imoney = StringToInt(money);
 
	if ((target_count = ProcessTargetString(
			target,
			0,
			target_list,
			MAXPLAYERS,
			0,
			target_name,
			sizeof(target_name),
			tn_is_ml)) <= 0)
	{
		ReplyToTargetError(client, target_count);
		return Plugin_Handled;
	}
    
	for (new i = 0; i < target_count; i++)
	{
		if (IsClientInGame(target_list[i]))
		{
			GivePlayerMoney(target_list[i], -1, imoney);
			PrintToChat(target_list[i], "You have received %d credits from the admin.", imoney);
		}
	}
        
	return Plugin_Continue;
}

MainMenu(client) 
{
	if (hSQL == INVALID_HANDLE)
	{
		ConnectSQL();
		PrintToChat(client, "Economy system is loading - try again in a few minutes.");
		
		return;
	}

	if (!IsValidPlayer(client))
		return;

	decl String:auth[32];
	GetClientAuthString(client, auth, sizeof(auth));

	decl String:query[255];
	Format(query, sizeof(query), "SELECT money FROM player WHERE auth = '%s'", auth);

	SQL_TQuery(hSQL, T_EconomyMenu, query, client, DBPrio_Normal);
}

CreateUser(client, money = DEFAULT_MONEY_AMOUNT)
{
	if (hSQL == INVALID_HANDLE)
	{
		ConnectSQL();
		return;
	}
	
	if (IsValidPlayer(client))
	{
		decl String:auth[32];
		GetClientAuthString(client, auth, sizeof(auth));

		decl String:query[255];
		Format(query, sizeof(query), "INSERT INTO player (auth, money) VALUES ('%s', %d)", auth, money);

		SQL_TQuery(hSQL, T_NullCallback, query, client, DBPrio_High);	
	}
}

public T_EconomyMenu(Handle:owner, Handle:hndl, const String:error[], any:client)
{
	if (owner == INVALID_HANDLE)
	{
		reconnectcounter++;
		ConnectSQL();
		
		return -1;
	}

	new bool:fetch = SQL_FetchRow(hndl);

	if (IsValidPlayer(client))
	{
		if (!fetch)
		{
			if (databaseCounter[client] >= 5)
			{
				PrintToChat(client, "Sorry, there's an issue with our database. We'll fix this as soon as possible.");
				databaseCounter[client] = 0;
				
				return 0;
			}

			decl String:auth[32];
			GetClientAuthString(client, auth, sizeof(auth));

			CreateUser(client);
	
			databaseCounter[client]++;

			decl String:query[255];
			Format(query, sizeof(query), "SELECT money FROM player WHERE auth = '%s'", auth);
			
			SQL_TQuery(hSQL, T_EconomyMenu, query, client, DBPrio_Normal);
			
			return 0;
		}
		else
		{
			decl String:currentMoneyText[64];
			Format(currentMoneyText, sizeof(currentMoneyText), "You have %d Credits.\n \n", GetSQLDataInt(hndl, "money"));

			new Handle:menu = CreateMenu(MainMenuSelectHandle);
			
			SetMenuTitle(menu, currentMoneyText);
			
			for (new category = 0; category < categoryCount; category++)
			{
				decl String:text[255];
				Format(text, sizeof(text), "%s", categories[category][CategoryName]);
				
				AddMenuItem(menu, categories[category][CategoryId], text);
			}
	
			AddMenuItem(menu, "inventory", "Inventory");
			
			new AdminId:admin = GetUserAdmin(client);
			
			if (admin != INVALID_ADMIN_ID)
			{
				if (GetAdminFlag(admin, Admin_Root))
					AddMenuItem(menu, "give_credits", "[Admin] Give Credits");
			}
			
			AddMenuItem(menu, "howtodonate", "How to donate?");
			
			SetMenuExitButton(menu, true);
			DisplayMenu(menu, client, MENU_TIME_FOREVER);
					
			databaseCounter[client] = 0;
			return 1;
		}
	}

	return 0;
}

public MainMenuSelectHandle(Handle:menu, MenuAction:action, client, slot)
{
	switch (action)
	{
		case MenuAction_Select:
		{
			new String:option[64];
		
			if (GetMenuItem(menu, slot, option, sizeof(option)))
			{
				if (StrEqual(option, "inventory"))
				{
					OpenInventory(client);
				}
				else if (StrEqual(option, "give_credits"))
				{
					OpenGiftMenu(client);
				}
				else if (StrEqual(option, "howtodonate"))
				{
					HowToDonate(client);
				}
				else
				{
					OpenShopCategory(client, option);
				}
			}
		}
		case MenuAction_End:
		{
			CloseHandle(menu);
		}
	}
}

HowToDonate(client)
{
	new Handle:menu = CreatePanel();
	SetPanelTitle(menu, "How To Donate?\n \n");
	
	for (new i = 0; i < howToDonateLinesCount; i++)
	{
		DrawPanelText(menu, howToDonate[i]);
	}
	
	DrawPanelText(menu, " ");
	DrawPanelItem(menu, "Back");
	SendPanelToClient(menu, client, HowToDonateHandle, MENU_TIME_FOREVER);	
}

public HowToDonateHandle(Handle:menu, MenuAction:action, client, slot)
{
	MainMenu(client);
}

public T_CreateUserCallback(Handle:owner, Handle:hndl, const String:error[], any:client)
{
	if (owner == INVALID_HANDLE)
	{
		reconnectcounter++;
		ConnectSQL();
		
		return -1;
	}

	new bool:fetch = SQL_FetchRow(hndl);

	if (IsValidPlayer(client))
	{
		if (!fetch)
		{
			decl String:auth[32];
			GetClientAuthString(client, auth, sizeof(auth));

			CreateUser(client);
		}

		return 1;
	}

	return 0;
}

public GetPlayerMoney(client)
{
	if (hSQL == INVALID_HANDLE)
	{
		ConnectSQL();
		return 0;
	}
	
	decl String:auth[32];
	GetClientAuthString(client, auth, sizeof(auth));

	decl String:query[255];
	Format(query, sizeof(query), "SELECT money FROM player WHERE auth = '%s'", auth);
	
	SQL_LockDatabase(hSQL);

	new Handle:hQuery = SQL_Query(hSQL, query);

	if (hQuery == INVALID_HANDLE)
	{
		SQL_UnlockDatabase(hSQL);
		return false;
	}

	SQL_UnlockDatabase(hSQL); 
	
	if (SQL_GetRowCount(hQuery) == 0)
	{
		CreateUser(client);
		return DEFAULT_MONEY_AMOUNT;
	}

	SQL_FetchRow(hQuery);
	return SQL_FetchInt(hQuery, 0);
}

public GivePlayerMoney(client, from, amount)
{
	if (hSQL == INVALID_HANDLE)
	{
		ConnectSQL();
		return;
	}
	
	decl String:auth[32];
	GetClientAuthString(client, auth, sizeof(auth));

	decl String:query[255];
	Format(query, sizeof(query), "INSERT INTO player (auth, money) values ('%s', %d) ON DUPLICATE KEY UPDATE money = money + %d", auth, DEFAULT_MONEY_AMOUNT, amount);

	SQL_TQuery(hSQL, T_NullCallback, query, client, DBPrio_Normal);

	if (from != -1)
		GivePlayerMoney(from, -1, -amount);
}

public T_NullCallback(Handle:owner, Handle:hndl, const String:error[], any:client)
{
	if (owner == INVALID_HANDLE)
	{
		reconnectcounter++;
		ConnectSQL();
		
		return -1;
	}

	return 1;
}

GetSQLDataInt(Handle:query, const String:columnname[])
{
	new column;
	SQL_FieldNameToNum(query, columnname, column);

	return SQL_FetchInt(query, column);
}

OpenShopCategory(client, String:categoryId[])
{
	strcopy(lastCategoryId[client], 64, categoryId);
	
	new Handle:menu = CreateMenu(ShopCategoryMenuSelectHandle);
	
	decl String:title[255];

	for (new category = 0; category < categoryCount; category++)
	{
		if (StrEqual(categoryId, categories[category][CategoryId]))
		{
			Format(title, sizeof(title), "Shop - %s\n \n", categories[category][CategoryName]); 
			break;
		}
	}
	
	SetMenuTitle(menu, title);
	
	for (new item = 0; item < itemCount; item++)
	{
		if (StrEqual(categoryId, items[item][ItemCategoryId]))
		{
			decl String:text[255];
			Format(text, sizeof(text), "%s [Price: %d Credits]", items[item][ItemName], items[item][ItemPrice], items[item][ItemDescription]);

			AddMenuItem(menu, items[item][ItemId], text);
		}
	}
	
	SetMenuExitBackButton(menu, true);
	DisplayMenu(menu, client, MENU_TIME_FOREVER);	 
}

public ShopCategoryMenuSelectHandle(Handle:menu, MenuAction:action, client, slot)
{
	if (action == MenuAction_Select)
	{
		new String:itemId[64];
		
		if (GetMenuItem(menu, slot, itemId, sizeof(itemId)))
		{
			BuyItem(client, itemId);
		}
	}
	else if (action == MenuAction_Cancel)
	{
		MainMenu(client);
	}
	else if (action == MenuAction_End)
	{
		CloseHandle(menu);
	}
}

BuyItem(client, String:itemId[])
{
	for (new item = 0; item < itemCount; item++)
	{
		if (StrEqual(items[item][ItemId], itemId))
		{
			if (items[item][ItemPrice] > GetPlayerMoney(client))
			{
				PrintToChat(client, "Sorry, you don't have enough credits to buy this item.");
			}
			else
			{
				GivePlayerMoney(client, -1, -items[item][ItemPrice]);
				AddItem(client, itemId);
			}
			break;	
		}
	} 
}

AddItem(client, String:itemId[])
{
	decl String:auth[32];
	GetClientAuthString(client, auth, sizeof(auth));

	decl String:query[255];
	Format(query, sizeof(query), "SELECT item_id FROM playeritem WHERE auth = '%s' AND item_id = '%s'", auth, itemId);

	new Handle:pack = CreateDataPack();
	WritePackCell(pack, client);
	WritePackString(pack, itemId);
	
	SQL_TQuery(hSQL, T_BuyItemCallback, query, pack, DBPrio_Normal);
}

public T_BuyItemCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE)
	{
		reconnectcounter++;
		ConnectSQL();
		
		return -1;
	}

	new Handle:pack = Handle:data;
	ResetPack(pack);
	
	new client = ReadPackCell(pack);
	
	new String:item_id[64];
	ReadPackString(pack, item_id, sizeof(item_id));
	
	CloseHandle(pack);
		
	decl String:auth[32];
	GetClientAuthString(client, auth, sizeof(auth));
	
	decl String:query[255];

	if (SQL_GetRowCount(hndl) == 0)
	{
		Format(query, sizeof(query), "INSERT INTO playeritem (auth, item_id, count) VALUES('%s', '%s', 1)", auth, item_id);	 
	}
	else
	{
		Format(query, sizeof(query), "UPDATE playeritem SET count = count + 1 WHERE auth = '%s' AND item_id = '%s'", auth, item_id);
	}
	
	SQL_TQuery(hSQL, T_BuyItemFinishCallback, query, client, DBPrio_Normal);
	return 1;
}

public T_BuyItemFinishCallback(Handle:owner, Handle:hndl, const String:error[], any:client)
{
	if (owner == INVALID_HANDLE)
	{
		reconnectcounter++;
		ConnectSQL();
		
		return -1;
	}

	PrintToChat(client, "Thank you for your purcashe! You can now go to your inventory and use your new item. Enjoy!");
	MainMenu(client);

	return 1;	
}

UseItem(client, String:itemId[])
{
	decl String:auth[32];
	GetClientAuthString(client, auth, sizeof(auth));

	decl String:query[255];
	Format(query, sizeof(query), "SELECT count FROM playeritem WHERE auth = '%s' AND item_id = '%s'", auth, itemId);

	new Handle:pack = CreateDataPack();
	WritePackCell(pack, client);
	WritePackString(pack, itemId);
	
	SQL_TQuery(hSQL, T_UseItemCallback, query, pack, DBPrio_Normal);
}

public T_UseItemCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE)
	{
		reconnectcounter++;
		ConnectSQL();
		
		return -1;
	}

	new Handle:pack = Handle:data;
	ResetPack(pack);
	
	new client = ReadPackCell(pack);
	
	new String:item_id[64];
	ReadPackString(pack, item_id, sizeof(item_id));
			
	decl String:auth[32];
	GetClientAuthString(client, auth, sizeof(auth));
	
	decl String:query[255];

	if (SQL_FetchRow(hndl))
	{
		if (SQL_FetchInt(hndl, 0) == 1)
		{
			Format(query, sizeof(query), "DELETE FROM playeritem WHERE auth = '%s' AND item_id = '%s'", auth, item_id);
		}
		else
		{
			Format(query, sizeof(query), "UPDATE playeritem SET count = count - 1 WHERE auth = '%s' AND item_id = '%s'", auth, item_id);
		}
	}
		 
	SQL_TQuery(hSQL, T_UseItemFinishCallback, query, pack, DBPrio_Normal);
	return 1;	
}

public T_UseItemFinishCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE)
	{
		reconnectcounter++;
		ConnectSQL();
		
		return -1;
	}
	
	new Handle:pack = Handle:data;
	ResetPack(pack);
	
	new client = ReadPackCell(pack);
	
	new String:item_id[64];
	ReadPackString(pack, item_id, sizeof(item_id));
	
	CloseHandle(pack);
	
	for (new item = 0; item < itemCount; item++)
	{
		if (StrEqual(items[item][ItemId], item_id))
		{
			OpenInventoryCategory(client, items[item][ItemCategoryId]);
			break;
		}
	}
		
	return 1;	
}

OpenInventory(client) 
{
	new Handle:menu = CreateMenu(InventoryMenuSelectHandle);
	SetMenuTitle(menu, "Inventory\n \n");
	
	for (new category = 0; category < categoryCount; category++)
	{
		decl String:text[255];
		Format(text, sizeof(text), "%s", categories[category][CategoryName], categories[category][CategoryDescription]);
		
		AddMenuItem(menu, categories[category][CategoryId], text);
	}
	
	SetMenuExitBackButton(menu, true);
	DisplayMenu(menu, client, MENU_TIME_FOREVER);	 
}

public InventoryMenuSelectHandle(Handle:menu, MenuAction:action, client, slot)
{
	if (action == MenuAction_Select)
	{
		new String:categoryId[64];
		
		if (GetMenuItem(menu, slot, categoryId, sizeof(categoryId)))
		{
			OpenInventoryCategory(client, categoryId);
		}
	}
	else if (action == MenuAction_Cancel)
	{
		MainMenu(client);
	}
	else if (action == MenuAction_End)
	{
		CloseHandle(menu);
	}
}

OpenInventoryCategory(client, String:categoryId[]) 
{
	decl String:auth[32];
	GetClientAuthString(client, auth, sizeof(auth));

	decl String:query[255];
	Format(query, sizeof(query), "SELECT item_id, count FROM playeritem WHERE auth = '%s'", auth);

	new Handle:pack = CreateDataPack();
	WritePackCell(pack, client);
	WritePackString(pack, categoryId);
	
	SQL_TQuery(hSQL, T_OpenInventoryCallback, query, pack, DBPrio_Normal);
}

public T_OpenInventoryCallback(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if (owner == INVALID_HANDLE)
	{
		reconnectcounter++;
		ConnectSQL();

		return -1;
	}

	new Handle:pack = Handle:data;
	ResetPack(pack);
	
	new client = ReadPackCell(pack);
	
	new String:categoryId[64];
	ReadPackString(pack, categoryId, sizeof(categoryId));
	
	if (SQL_GetRowCount(hndl) == 0)
	{
		PrintToChat(client, "You have no items in your inventory.");
		MainMenu(client);
	}
	else
	{
		new Handle:menu = CreateMenu(InventoryMenu2SelectHandle);
		
		for (new category = 0; category < categoryCount; category++)
		{
			if (StrEqual(categories[category][CategoryId], categoryId))
			{
				SetMenuTitle(menu, "Inventory - %s\n \n", categories[category][CategoryName]);
				break;
			}
		}
		
		new String:item_id[64];
		
		while (SQL_FetchRow(hndl)) 
		{
			SQL_FetchString(hndl, 0, item_id, sizeof(item_id));
	
			new count = SQL_FetchInt(hndl, 1);
			
			for (new i = 0; i < itemCount; i++)
			{
				if (StrEqual(items[i][ItemId], item_id) && StrEqual(items[i][CategoryId], categoryId))
				{
					new String:prefix[5];
					
					if (IsToggled(client, items[i][ItemId]))
						prefix = "[X] ";
					
					new String:suffix[5];
					
					if (items[i][IsItemFinite] || count > 1)
						Format(suffix, sizeof(suffix), " [%d]", count);
					
					decl String:text[255];
					Format(text, sizeof(text), "%s%s%s", prefix, items[i][ItemName], suffix);
					
					AddMenuItem(menu, item_id, text);
					break;
				}
			}
		}
		
		SetMenuExitBackButton(menu, true);
		DisplayMenu(menu, client, MENU_TIME_FOREVER);
	}

	return 1;	
}

public InventoryMenu2SelectHandle(Handle:menu, MenuAction:action, client, slot)
{
	if (action == MenuAction_Select)
	{
		new String:itemId[64];
		
		if (GetMenuItem(menu, slot, itemId, sizeof(itemId)))
		{
			for (new item = 0; item < itemCount; item++)
			{
				if (StrEqual(items[item][ItemId], itemId))
				{
					Call_StartFunction(items[item][ItemCallbackPlugin], items[item][ItemCallback]);
					Call_PushCell(client);
					Call_PushString(items[item][ItemId]);
					
					new bool:result = false;
					Call_Finish(result);
						
					if (result && items[item][IsItemFinite])
					{
						UseItem(client, items[item][ItemId]);
					}
					else
					{
						OpenInventoryCategory(client, items[item][ItemCategoryId]);
					}

					break;
				}
			}
		}
	}
	else if (action == MenuAction_Cancel)
	{
		OpenInventory(client);
	}
	else if (action == MenuAction_End)
	{
		CloseHandle(menu);
	}
}

OpenGiftMenu(client)
{
	new Handle:menu = CreateMenu(Gift1MenuSelectHandle);
	SetMenuTitle(menu, "Give Credits To?\n \n");
	
	new String:name[64];
	new String:value[64];
	
	for (new i = 1; i <= MaxClients; i++)
	{
		if (IsValidPlayer(i) && GetClientName(i, name, sizeof(name)))
		{
			IntToString(i, value, sizeof(value));
			AddMenuItem(menu, value, name);
		}
	}
	
	SetMenuExitBackButton(menu, true);
	DisplayMenu(menu, client, MENU_TIME_FOREVER);	
}

public Gift1MenuSelectHandle(Handle:menu, MenuAction:action, client, slot)
{
	if (action == MenuAction_Select)
	{
		new String:item[64];
		
		if (GetMenuItem(menu, slot, item, sizeof(item)))
		{
			new client2 = StringToInt(item);
			OpenGift2Menu(client, client2);			
		}
	}
	else if (action == MenuAction_Cancel)
	{
		MainMenu(client);
	}
	else if (action == MenuAction_End)
	{
		CloseHandle(menu);
	}		
}

OpenGift2Menu(client, client2) 
{
	new Handle:menu = CreateMenu(Gift2MenuSelectHandle);
	SetMenuTitle(menu, "How much Credits?\n \n");
	
	new String:value[64];
	IntToString(client2, value, sizeof(value));
	
	AddMenuItem(menu, value, "50");
	AddMenuItem(menu, value, "100");
	AddMenuItem(menu, value, "500");
	AddMenuItem(menu, value, "1000");
	AddMenuItem(menu, value, "2000");
	AddMenuItem(menu, value, "5000");
	AddMenuItem(menu, value, "10000");
	AddMenuItem(menu, value, "50000");

	SetMenuExitBackButton(menu, true);
	DisplayMenu(menu, client, MENU_TIME_FOREVER);		
}

public Gift2MenuSelectHandle(Handle:menu, MenuAction:action, client, slot)
{
	if (action == MenuAction_Select)
	{
		new String:item[64];
		
		if (GetMenuItem(menu, slot, item, sizeof(item)))
		{
			new client2 = StringToInt(item);
			
			new amount = 0;
			
			switch (slot)
			{
				case 0:
				{
					amount = 50;
				}	
				case 1:
				{
					amount = 100;
				}
					
				case 2:
				{
					amount = 500;
				}
					
				case 3:
				{
					amount = 1000;
				}

				case 4:
				{
					amount = 2000;
				}				

				case 5:
				{
					amount = 5000;
				}
				case 6:
				{
					amount = 10000;
				}

				case 7:
				{
					amount = 50000;
				}				
			}
			
			GivePlayerMoney(client2, -1, amount);
		
			new String:name[64];
			GetClientName(client, name, sizeof(name));

			new String:name2[64];
			GetClientName(client2, name2, sizeof(name));
			
			PrintToChat(client, "You have successfully gave %d credits to %s.", amount, name2);
		}
	}
	else if (action == MenuAction_Cancel)
	{
		OpenGiftMenu(client);
	}
	else if (action == MenuAction_End)
	{
		CloseHandle(menu);
	}		
}

public Native_RegisterItemCategory(Handle:plugin, params)
{
 	new String:id[64];
	GetNativeString(1, id, sizeof(id));
	
	new String:name[64];
	GetNativeString(2, name, sizeof(name));	
	
	new String:description[64];
	GetNativeString(3, description, sizeof(description));
	
	RegisterItemCategory(id, name, description);
}

public Native_RegisterItem(Handle:plugin, params)
{
	new String:categoryId[64];
	GetNativeString(1, categoryId, sizeof(categoryId));
	
	new String:id[64];
	GetNativeString(2, id, sizeof(id));
	
	new String:name[64];
	GetNativeString(3, name, sizeof(name));	
	
	new String:description[64];
	GetNativeString(4, description, sizeof(description));
	
	new ItemUseCallback:callback = GetNativeCell(5);
	new price = GetNativeCell(6);
	new bool:isFinite = GetNativeCell(7);
	new bool:isTogglable = GetNativeCell(8);
	
	RegisterItem(categoryId, id, name, description, plugin, callback, price, isFinite, isTogglable);
}

public Native_GivePlayerMoney(Handle:plugin, params)
{
	new client = GetNativeCell(1);
	new from = GetNativeCell(2);
	new amount = GetNativeCell(3);

	GivePlayerMoney(client, from, amount);
}

public Native_GivePlayerItem(Handle:plugin, params)
{
	new client = GetNativeCell(1);
	
	new String:id[32];
	GetNativeString(2, id, sizeof(id));

	AddItem(client, id);
}

public Native_GetPrimaryToggled(Handle:plugin, params)
{
	new client = GetNativeCell(1);

	new String:categoryId[64];
	GetNativeString(2, categoryId, sizeof(categoryId));
	
	new String:itemId[64];
	new success = GetPrimaryToggled(client, categoryId, itemId);

	if (success)
	SetNativeString(3, itemId, 64);
	
	return success;
}

public Native_IsToggled(Handle:plugin, params)
{
	new client = GetNativeCell(1);
	
	new String:id[64];
	GetNativeString(2, id, sizeof(id));
	
	return IsToggled(client, id);
}

public Native_ToggleItem(Handle:plugin, params)
{
	new client = GetNativeCell(1);
	
	new String:id[64];
	GetNativeString(2, id, sizeof(id));
	
	new bool:value = GetNativeCell(3);
	
	ToggleItem(client, id, value);
}

public Native_ToggleItemCategory(Handle:plugin, params)
{
	new client = GetNativeCell(1);
	
	new String:id[64];
	GetNativeString(2, id, sizeof(id));
	
	new bool:value = GetNativeCell(3);
	
	ToggleItemCategory(client, id, value);
}

public Native_GetPlayerMoney(Handle:plugin, params)
{
	return GetPlayerMoney(GetNativeCell(1));
}