#if defined _economy_included
 #endinput
#endif
#define _economy_included

#pragma semicolon 1

#define PL_VERSION "1.0.0-dev"

functag public ItemUseCallback(client, String:id[]);

/**
 * Gives some money to a specific player.
 *
 * @param client	The client to give zold to.
 * @param from		That client that gives the zold (-1 = server)
 * @param amount	The amount of money to give to the player.
 *
 * @noreturn
 */
native Economy_GivePlayerMoney(client, from, zold);

/**
 * Fetches the amount of money a player currently has.
 *
 * @param client	Player index.
 *
 * @return The amount of money that the player has.
 */
native Economy_GetPlayerMoney(client);

/**
 * Adds an item to the market.
 *
 * @param categoryId	The category's unique id to add the item to. 
 * @param id 			A unique identity that represents the item (lowercase, no spaces).
 * @param name 			A friendly name for the item.
 * @param description	A long description about the item.
 * @param callback 		A callback that will be called when a player uses the item.
 * @param price			The price of the item.
 * @param isFinite		Whether the amount of the item reduces when a player uses it.
 * @param isTogglable	Can player toggle this item?
 * @param isTradeable
 * @param addToShop
 * 
 * @noreturn
 */
native Economy_RegisterItem(String:categoryId[], String:id[], String:name[], String:description[], ItemUseCallback:callback, price, bool:isFinite, bool:isTogglable, bool:isTradeable, bool:addToShop);

/**
 * Adds a category to the market.
 *
 * @param id 			A unique identity that represents the category (lowercase, no spaces).
 * @param name 			A friendly name for the category.
 * @param description	A long description about the category.
 *
 * @noreturn
 */
native Economy_RegisterItemCategory(String:id[], String:name[], String:description[]);

/**
 * Fetches the currently toggled item for a player in a specific category.
 *
 * @param client		Player index.
 * @param categoryId	The category's unique id. 
 * @param itemId		Buffer to store the item id.
 *
 * @return True on success, false otherwise.
 */
native bool:Economy_GetPrimaryToggled(client, String:categoryId[], String:itemId[]);

/**
 * Fetches the currently toggled item for a player in a specific category.
 *
 * @param client		Player index.
 * @param id			The item's unique id.
 *
 * @return True when the item is toggled, false otherwise.
 */
native bool:Economy_IsToggled(client, String:id[]);

/**
 * Toggles the item.
 *
 * @param client		Player index.
 * @param id			The item's unique id.
 * @param value 		The value to change to.
 *
 * @noreturn
 */
native Economy_ToggleItem(client, String:id[], bool:value);

/**
 * Toggles all items in this category.
 *
 * @param client		Player index.
 * @param id			The category's unique id.
 * @param value 		The value to change to.
 *
 * @noreturn
 */
native Economy_ToggleItemCategory(client, String:id[], bool:value);

public SharedPlugin:__pl_economy = 
{
	name = "economy",
	file = "economy.smx",
#if defined REQUIRE_PLUGIN
	required = 1,
#else
	required = 0,
#endif
};

public __pl_economy_SetNTVOptional()
{
	MarkNativeAsOptional("Zold_GivePlayerMoney");
	MarkNativeAsOptional("Zold_GetPlayerMoney");
	
	MarkNativeAsOptional("Zold_RegisterItem");
	MarkNativeAsOptional("Zold_RegisterItemCategory");
	
	MarkNativeAsOptional("Zold_GetPrimaryToggled");
	MarkNativeAsOptional("Zold_IsToggled");
	MarkNativeAsOptional("Zold_ToggleItem");
	MarkNativeAsOptional("Zold_ToggleItemCategory");
}